@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2016	PgPool Global Development Group
+ * Copyright (c) 2003-2012	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -75,8 +75,6 @@ static void init_system_db_connection(void);
 static bool connect_using_existing_connection(POOL_CONNECTION *frontend,
 											  POOL_CONNECTION_POOL *backend,
 											  StartupPacket *sp);
-static void free_persisten_db_connection_memory(POOL_CONNECTION_POOL_SLOT *cp);
-
 /*
  * non 0 means SIGTERM(smart shutdown) or SIGINT(fast shutdown) has arrived
  */
@@ -85,9 +83,6 @@ volatile sig_atomic_t exit_request = 0;
 static int idle;		/* non 0 means this child is in idle state */
 static int accepted = 0;
 
-static int child_inet_fd = 0;
-static int child_unix_fd = 0;
-
 extern int myargc;
 extern char **myargv;
 
@@ -95,6 +90,9 @@ char remote_ps_data[NI_MAXHOST];		/* used for set_ps_display */
 
 volatile sig_atomic_t got_sighup = 0;
 
+char remote_host[NI_MAXHOST];	/* client host */
+char remote_port[NI_MAXSERV];	/* client port */
+
 /*
 * child main loop
 */
@@ -134,11 +132,6 @@ void do_child(int unix_fd, int inet_fd)
 		pool_set_nonblock(inet_fd);
 	}
 #endif
-	if (inet_fd)
-	{
-		child_inet_fd = inet_fd;
-	}
-	child_unix_fd = unix_fd;
 
 	/* Initialize my backend status */
 	pool_initialize_private_backend_status();
@@ -163,15 +156,6 @@ void do_child(int unix_fd, int inet_fd)
 		child_exit(1);
 	}
 
-	/*
-	 * Open pool_passwd in child process.  This is necessary to avoid the
-	 * file descriptor race condition reported in [pgpool-general: 1141].
-	 */
-	if (strcmp("", pool_config->pool_passwd))
-	{
-		pool_reopen_passwd_file();
-	}
-
 	timeout.tv_sec = pool_config->child_life_time;
 	timeout.tv_usec = 0;
 
@@ -256,7 +240,6 @@ void do_child(int unix_fd, int inet_fd)
 		{
 			pool_debug("SSLRequest from client");
 			pool_ssl_negotiate_serverclient(frontend);
-			pool_free_startup_packet(sp);
 			goto retry_startup;
 		}
 
@@ -368,12 +351,6 @@ void do_child(int unix_fd, int inet_fd)
 		 */
 		pool_init_session_context(frontend, backend);
 
-		/*
-		 * Set protocol versions
-		 */
-		pool_set_major_version(sp->major);
-		pool_set_minor_version(sp->minor);
-
 		/* Mark this connection pool is conncted from frontend */
 		pool_coninfo_set_frontend_connected(pool_get_process_context()->proc_id, pool_pool_index());
 
@@ -390,15 +367,12 @@ void do_child(int unix_fd, int inet_fd)
 			{
 				/* client exits */
 				case POOL_END:
-				case POOL_END_WITH_FRONTEND_ERROR:
 					/*
 					 * do not cache connection if:
-					 * frontend abnormally exits or
 					 * pool_config->connection_cahe == 0 or
 					 * database name is template0, template1, postgres or regression
 					 */
-					if (status == POOL_END_WITH_FRONTEND_ERROR ||
-						pool_config->connection_cache == 0 ||
+					if (pool_config->connection_cache == 0 ||
 						!strcmp(sp->database, "template0") ||
 						!strcmp(sp->database, "template1") ||
 						!strcmp(sp->database, "postgres") ||
@@ -513,9 +487,6 @@ static POOL_CONNECTION *do_accept(int unix_fd, int inet_fd, struct timeval *time
 	struct timeval *timeoutval;
 	struct timeval tv1, tv2, tmback = {0, 0};
 
-	char remote_host[NI_MAXHOST];
-	char remote_port[NI_MAXSERV];
-
 	set_ps_display("wait for connection request", false);
 
 	/* Destroy session context for just in case... */
@@ -670,8 +641,7 @@ static POOL_CONNECTION *do_accept(int unix_fd, int inet_fd, struct timeval *time
 		if (pool_config->enable_pool_hba)
 		{
 			load_hba(get_hba_file_name());
-			if (strcmp("", pool_config->pool_passwd))
-				pool_reopen_passwd_file();
+			pool_reopen_passwd_file();
 		}
 		if (pool_config->parallel_mode)
 			pool_memset_system_db_info(system_db_info->info);
@@ -854,10 +824,6 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 	/* read startup packet length */
 	if (pool_read(cp, &len, sizeof(len)))
 	{
-		pool_error("read_startup_packet: incorrect packet length (%d)", len);
-		pool_free_startup_packet(sp);
-		alarm(0);
-		pool_signal(SIGALRM, SIG_IGN);
 		return NULL;
 	}
 	len = ntohl(len);
@@ -1009,28 +975,6 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 			return NULL;
 	}
 
-	/* Check a user name was given. */
-	if (sp->major != 1234 &&
-	    (sp->user == NULL || sp->user[0] == '\0'))
-	{
-		pool_send_fatal_message(cp, sp->major, "28000",
-		                        "no PostgreSQL user name specified in startup packet",
-								"",
-								"",
-								__FILE__, __LINE__);
-		pool_error("read_startup_packet: no PostgreSQL user name specified in startup packet");
-		pool_free_startup_packet(sp);
-		alarm(0);
-		pool_signal(SIGALRM, SIG_IGN);
-		return NULL;
-	}
-
-	/* The database defaults to ther user name. */
-	if (sp->database == NULL || sp->database[0] == '\0')
-	{
-		sp->database = strdup(sp->user);
-	}
-
 	pool_debug("Protocol Major: %d Minor: %d database: %s user: %s",
 			   sp->major, sp->minor, sp->database, sp->user);
 	alarm(0);
@@ -1039,8 +983,8 @@ static StartupPacket *read_startup_packet(POOL_CONNECTION *cp)
 }
 
 /*
- * send startup packet
- */
+* send startup packet
+*/
 int send_startup_packet(POOL_CONNECTION_POOL_SLOT *cp)
 {
 	int len;
@@ -1053,7 +997,7 @@ int send_startup_packet(POOL_CONNECTION_POOL_SLOT *cp)
 /*
  * Reuse existing connection
  */
-static bool connect_using_existing_connection(POOL_CONNECTION *frontend,
+static bool connect_using_existing_connection(POOL_CONNECTION *frontend, 
 											  POOL_CONNECTION_POOL *backend,
 											  StartupPacket *sp)
 {
@@ -1296,14 +1240,6 @@ static RETSIGTYPE die(int sig)
 	switch (sig)
 	{
 		case SIGTERM:	/* smart shutdown */
-			/* Refuse further requests by closing listen socket */
-			if (child_inet_fd)
-			{
-				pool_log("die: close listen socket");
-				close(child_inet_fd);
-			}
-			close(child_unix_fd);
-
 			if (idle == 0)
 			{
 				pool_debug("child receives smart shutdown request but it's not in idle state");
@@ -1466,9 +1402,13 @@ void child_exit(int code)
 			pool_close(pool_system_db_connection()->con);
 	}
 
+	if (pool_config->memory_cache_enabled && !pool_is_shmem_cache())
+	{
+		memcached_disconnect();
+	}
+
 	/* let backend know now we are exiting */
-	if (pool_connection_pool)
-		send_frontend_exits();
+	send_frontend_exits();
 
 	exit(code);
 }
@@ -1477,7 +1417,7 @@ void child_exit(int code)
  * create a persistent connection
  */
 POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
-	char *hostname, int port, char *dbname, char *user, char *password)
+	char *hostname, int port, char *dbname, char *user, char *password, bool retry)
 {
 	POOL_CONNECTION_POOL_SLOT *cp;
 	int fd;
@@ -1506,7 +1446,6 @@ POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
 	if (startup_packet == NULL)
 	{
 		pool_error("make_persistent_db_connection: could not allocate memory");
-		free_persisten_db_connection_memory(cp);
 		return NULL;
 	}
 	memset(startup_packet, 0, sizeof(*startup_packet));
@@ -1517,18 +1456,16 @@ POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
 	 */
 	if (*hostname == '/')
 	{
-		fd = connect_unix_domain_socket_by_port(port, hostname, TRUE);
+		fd = connect_unix_domain_socket_by_port(port, hostname, retry);
 	}
 	else
 	{
-		fd = connect_inet_domain_socket_by_port(hostname, port, TRUE);
+		fd = connect_inet_domain_socket_by_port(hostname, port, retry);
 	}
 
 	if (fd < 0)
 	{
 		pool_error("make_persistent_db_connection: connection to %s(%d) failed", hostname, port);
-		free_persisten_db_connection_memory(cp);
-		free(startup_packet);
 		return NULL;
 	}
 
@@ -1545,9 +1482,6 @@ POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
 	if (len1 >= (sizeof(startup_packet->data)-len))
 	{
 		pool_error("make_persistent_db_connection: too long user name");
-		pool_close(cp->con);
-		free_persisten_db_connection_memory(cp);
-		free(startup_packet);
 		return NULL;
 	}
 
@@ -1556,9 +1490,6 @@ POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
 	if (len1 >= (sizeof(startup_packet->data)-len))
 	{
 		pool_error("make_persistent_db_connection: too long user name");
-		pool_close(cp->con);
-		free_persisten_db_connection_memory(cp);
-		free(startup_packet);
 		return NULL;
 	}
 
@@ -1567,9 +1498,6 @@ POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
 	if (len1 >= (sizeof(startup_packet->data)-len))
 	{
 		pool_error("make_persistent_db_connection: too long database name");
-		pool_close(cp->con);
-		free_persisten_db_connection_memory(cp);
-		free(startup_packet);
 		return NULL;
 	}
 	len += len1;
@@ -1579,9 +1507,6 @@ POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
 	if (cp->sp == NULL)
 	{
 		pool_error("make_persistent_db_connection: could not allocate memory");
-		pool_close(cp->con);
-		free_persisten_db_connection_memory(cp);
-		free(startup_packet);
 		return NULL;
 	}
 
@@ -1593,16 +1518,12 @@ POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
 	if (cp->sp->database == NULL)
 	{
 		pool_error("make_persistent_db_connection: could not allocate memory");
-		pool_close(cp->con);
-		free_persisten_db_connection_memory(cp);
 		return NULL;
 	}
 	cp->sp->user = strdup(user);
 	if (cp->sp->user == NULL)
 	{
 		pool_error("make_persistent_db_connection: could not allocate memory");
-		pool_close(cp->con);
-		free_persisten_db_connection_memory(cp);
 		return NULL;
 	}
 
@@ -1613,8 +1534,6 @@ POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
 	if (status)
 	{
 		pool_error("make_persistent_db_connection: send_startup_packet failed");
-		pool_close(cp->con);
-		free_persisten_db_connection_memory(cp);
 		return NULL;
 	}
 
@@ -1624,37 +1543,12 @@ POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
 	if (s_do_auth(cp, password))
 	{
 		pool_error("make_persistent_db_connection: s_do_auth failed");
-		pool_close(cp->con);
-		free_persisten_db_connection_memory(cp);
 		return NULL;
 	}
 
 	return cp;
 }
 
-/*
- * Free memory of POOL_CONNECTION_POOL_SLOT.  Should only be used in
- * make_persistent_db_connection and discard_persistent_db_connection.
- */
-void free_persisten_db_connection_memory(POOL_CONNECTION_POOL_SLOT *cp)
-{
-	if (!cp)
-		return;
-	if (!cp->sp)
-	{
-		free(cp);
-		return;
-	}
-	if (cp->sp->startup_packet)
-		free(cp->sp->startup_packet);
-	if (cp->sp->database)
-		free(cp->sp->database);
-	if (cp->sp->user)
-		free(cp->sp->user);
-	free(cp->sp);
-	free(cp);
-}
-
 /*
  * Discard connection and memory allocated by
  * make_persistent_db_connection().
@@ -1682,7 +1576,11 @@ void discard_persistent_db_connection(POOL_CONNECTION_POOL_SLOT *cp)
 	pool_unset_nonblock(cp->con->fd);
 
 	pool_close(cp->con);
-	free_persisten_db_connection_memory(cp);
+	free(cp->sp->startup_packet);
+	free(cp->sp->database);
+	free(cp->sp->user);
+	free(cp->sp);
+	free(cp);
 }
 
 /*
@@ -2068,9 +1966,8 @@ static void init_system_db_connection(void)
 {	
 	if (pool_config->parallel_mode || pool_config->enable_query_cache)
 	{
-		int nRet;
-		nRet = system_db_connect();
-		if (nRet || PQstatus(system_db_info->pgconn) != CONNECTION_OK)
+		system_db_connect();
+		if (PQstatus(system_db_info->pgconn) != CONNECTION_OK)
 		{
 			pool_error("Could not make persistent libpq system DB connection");
 		}
@@ -2079,7 +1976,7 @@ static void init_system_db_connection(void)
 																   pool_config->system_db_port,
 																   pool_config->system_db_dbname,
 																   pool_config->system_db_user,
-																   pool_config->system_db_password);
+																   pool_config->system_db_password, false);
 		if (system_db_info->connection == NULL)
 		{
 			pool_error("Could not make persistent system DB connection");
