@@ -459,24 +459,22 @@ static int send_cached_messages(POOL_CONNECTION *frontend, const char *qcache, i
 	int msg = 0;
 	int i = 0;
 	int is_prepared_stmt = 0;
+	int len;
+	const char *p;
 
 	while (i < qcachelen)
 	{
 		char tmpkind;
 		int tmplen;
-		char tmpbuf[MAX_VALUE];
 
 		tmpkind = qcache[i];
-
-		i += 1;
+		i++;
 
 		memcpy(&tmplen, qcache+i, sizeof(tmplen));
 		i += sizeof(tmplen);
-
-		tmplen = ntohl(tmplen);
-
-		memcpy(tmpbuf, qcache+i, tmplen - sizeof(tmplen));
-		i += tmplen - sizeof(tmplen);
+		len = ntohl(tmplen);
+		p = qcache + i;
+		i += len - sizeof(tmplen);
 
 		/* No need to cache PARSE and BIND responses */
 		if (tmpkind == '1' || tmpkind == '2')
@@ -495,8 +493,8 @@ static int send_cached_messages(POOL_CONNECTION *frontend, const char *qcache, i
 		}
 
 		/* send message to frontend */
-		pool_debug("send_cached_messages: %c len: %d", tmpkind, tmplen);
-		send_message(frontend, tmpkind, tmplen, tmpbuf);
+		pool_debug("send_cached_messages: %c len: %d", tmpkind, len);
+		send_message(frontend, tmpkind, len, p);
 
 		msg++;
 	}
@@ -645,6 +643,8 @@ POOL_STATUS pool_fetch_from_memory_cache(POOL_CONNECTION *frontend,
  */
 bool pool_is_likely_select(char *query)
 {
+	bool do_continue = false;
+
 	if (query == NULL)
 		return false;
 
@@ -654,27 +654,61 @@ bool pool_is_likely_select(char *query)
 		while (*query && isspace(*query))
 			query++;
 	}
+	if (! *query) { return false; }
 
 	/*
 	 * Get rid of head comment.
 	 * It is sure that the query is in correct format, because the parser
 	 * has rejected bad queries such as the one with not-ended comment.
 	 */
-	if (*query && !strncmp(query, "/*", 2))
+	while (*query)
 	{
-		while (*query && strncmp(query, "*/", 2))
-			query++;
-
-		if (*query)
-			query += 2;
-
+		/* Ignore spaces and return marks */
+		do_continue = false;
 		while (*query && isspace(*query))
+		{
 			query++;
-	}
+			do_continue = true;
+		}
+		if (do_continue) { continue; }
 
-	if (!strncasecmp(query, "SELECT", 6) || !strncasecmp(query, "WITH", 4))
-	{
-		return true;
+		while (*query && !strncmp(query, "\n", 2))
+		{
+			query++;
+			do_continue = true;
+		}
+		if (do_continue)
+		{
+			query += 2;
+			continue;
+		}
+
+		/* Ignore comments like C */
+		if (!strncmp(query, "/*", 2))
+		{
+			while (*query && strncmp(query, "*/", 2))
+				query++;
+
+			query += 2;
+			continue;
+		}
+
+		/* Ignore SQL comments */
+		if (!strncmp(query, "--", 2))
+		{
+			while (*query && strncmp(query, "\n", 2))
+				query++;
+
+			query += 2;
+			continue;
+		}
+
+		if (!strncasecmp(query, "SELECT", 6) || !strncasecmp(query, "WITH", 4))
+		{
+			return true;
+		}
+
+		query++;
 	}
 
 	return false;
@@ -1218,6 +1252,7 @@ static void pool_add_table_oid_map(POOL_CACHEKEY *cachekey, int num_table_oids,
 		int fd;
 		int oid = table_oids[i];
 		int sts;
+		struct flock fl;
 
 		/*
 		 * Create or open each memqcache_oiddir/database_oid/table_oid
@@ -1229,10 +1264,16 @@ static void pool_add_table_oid_map(POOL_CACHEKEY *cachekey, int num_table_oids,
 					   path, strerror(errno));
 			return;
 		}
-		sts = flock(fd, LOCK_EX);
+
+		fl.l_type   = F_WRLCK;
+		fl.l_whence = SEEK_SET;
+		fl.l_start  = 0;        	/* Offset from l_whence         */
+		fl.l_len    = 0;        	/* length, 0 = to EOF           */
+
+		sts = fcntl(fd, F_SETLKW, &fl);
 		if (sts == -1)
 		{
-			pool_error("pool_add_table_oid_map: failed to flock %s. reason:%s",
+			pool_error("pool_add_table_oid_map: failed to lock %s. reason:%s",
 					   path, strerror(errno));
 			close(fd);
 			return;
@@ -1392,6 +1433,7 @@ static void pool_invalidate_query_cache(int num_table_oids, int *table_oid, bool
 		int fd;
 		int oid = table_oid[i];
 		int sts;
+		struct flock fl;
 
 		/*
 		 * Open each memqcache_oiddir/database_oid/table_oid
@@ -1407,10 +1449,16 @@ static void pool_invalidate_query_cache(int num_table_oids, int *table_oid, bool
 					   path, strerror(errno));
 			return;
 		}
-		sts = flock(fd, LOCK_EX);
+
+		fl.l_type   = F_RDLCK;
+		fl.l_whence = SEEK_SET;
+		fl.l_start  = 0;        	/* Offset from l_whence         */
+		fl.l_len    = 0;        	/* length, 0 = to EOF           */
+
+		sts = fcntl(fd, F_SETLKW, &fl);
 		if (sts == -1)
 		{
-			pool_error("pool_invalidate_query_cache: failed to flock %s. reason:%s",
+			pool_error("pool_invalidate_query_cache: failed to lock %s. reason:%s",
 					   path, strerror(errno));
 			close(fd);
 			return;
