@@ -6,7 +6,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2016	PgPool Global Development Group
+ * Copyright (c) 2003-2012	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -95,11 +95,13 @@ void pool_start_query(POOL_QUERY_CONTEXT *query_context, char *query, int len, N
 		session_context = pool_get_session_context();
 		query_context->original_length = len;
 		query_context->rewritten_length = -1;
-		query_context->original_query = query;
+		query_context->original_query = pstrdup(query);
 		query_context->rewritten_query = NULL;
 		query_context->parse_tree = node;
 		query_context->virtual_master_node_id = my_master_node_id;
-		query_context->num_original_params = -1;
+		query_context->is_cache_safe = false;
+		if (pool_config->memory_cache_enabled)
+			query_context->temp_cache = pool_create_temp_query_cache(query);
 		pool_set_query_in_progress();
 		session_context->query_context = query_context;
 	}
@@ -116,7 +118,7 @@ void pool_set_node_to_be_sent(POOL_QUERY_CONTEXT *query_context, int node_id)
 		return;
 	}
 
-	if (node_id < 0 || node_id >= MAX_NUM_BACKENDS)
+	if (node_id < 0 || node_id > MAX_NUM_BACKENDS)
 	{
 		pool_error("pool_set_node_to_be_sent: invalid node id:%d", node_id);
 		return;
@@ -138,7 +140,7 @@ void pool_unset_node_to_be_sent(POOL_QUERY_CONTEXT *query_context, int node_id)
 		return;
 	}
 
-	if (node_id < 0 || node_id >= MAX_NUM_BACKENDS)
+	if (node_id < 0 || node_id > MAX_NUM_BACKENDS)
 	{
 		pool_error("pool_unset_node_to_be_sent: invalid node id:%d", node_id);
 		return;
@@ -169,14 +171,6 @@ void pool_clear_node_to_be_sent(POOL_QUERY_CONTEXT *query_context)
 void pool_setall_node_to_be_sent(POOL_QUERY_CONTEXT *query_context)
 {
 	int i;
-	POOL_SESSION_CONTEXT *sc;
-
-	sc = pool_get_session_context();
-	if (!sc)
-	{
-		pool_error("pool_setall_node_to_be_sent: no session context");
-		return;
-	}
 
 	if (!query_context)
 	{
@@ -188,20 +182,7 @@ void pool_setall_node_to_be_sent(POOL_QUERY_CONTEXT *query_context)
 	{
 		if (private_backend_status[i] == CON_UP ||
 			(private_backend_status[i] == CON_CONNECT_WAIT))
-		{
-			/*
-			 * In streaming replication mode, if the node is not
-			 * primary node nor load balance node, there's no point to
-			 * send query.
-			 */
-			if (pool_config->master_slave_mode &&
-				!strcmp(pool_config->master_slave_sub_mode, MODE_STREAMREP) &&
-				i != PRIMARY_NODE_ID && i != sc->load_balance_node_id)
-			{
-				continue;
-			}
 			query_context->where_to_send[i] = true;
-		}
 	}
 	return;
 }
@@ -247,7 +228,7 @@ bool pool_is_node_to_be_sent(POOL_QUERY_CONTEXT *query_context, int node_id)
 		return false;
 	}
 
-	if (node_id < 0 || node_id >= MAX_NUM_BACKENDS)
+	if (node_id < 0 || node_id > MAX_NUM_BACKENDS)
 	{
 		pool_error("pool_is_node_to_be_sent: invalid node id:%d", node_id);
 		return false;
@@ -293,23 +274,7 @@ int pool_virtual_master_db_node_id(void)
 
 	if (sc->query_context)
 	{
-		int node_id = sc->query_context->virtual_master_node_id;
-
-		if (STREAM)
-		{
-			 /*
-			  * Make sure that virtual_master_node_id is either primary node
-			  * id or load balance node id.  If not, it is likely that
-			  * virtual_master_node_id is not set up yet. Let's use the
-			  * primary node id.
-			  */
-			if (node_id != sc->load_balance_node_id && node_id != PRIMARY_NODE_ID)
-			{
-				node_id = PRIMARY_NODE_ID;
-			}
-		}
-
-		return node_id;
+		return sc->query_context->virtual_master_node_id;
 	}
 
 	/*
@@ -390,11 +355,11 @@ void pool_where_to_send(POOL_QUERY_CONTEXT *query_context, char *query, Node *no
 	else if (MASTER_SLAVE)
 	{
 		POOL_DEST dest;
-		POOL_MEMORY_POOL *old_context = pool_memory;
+		POOL_MEMORY_POOL *old_context;
 
-		pool_memory = query_context->memory_context;
+		old_context = pool_memory_context_switch_to(query_context->memory_context);
 		dest = send_to_where(node, query);
-		pool_memory = old_context;
+		pool_memory_context_switch_to(old_context);
 
 		pool_debug("send_to_where: %d query: %s", dest, query);
 
@@ -475,7 +440,7 @@ void pool_where_to_send(POOL_QUERY_CONTEXT *query_context, char *query, Node *no
 					 * If temporary table is used in the SELECT,
 					 * we prefer to send to the primary.
 					 */
-					else if (pool_has_temp_table(node))
+					else if (pool_config->check_temp_table && pool_has_temp_table(node))
 					{
 						pool_set_node_to_be_sent(query_context, PRIMARY_NODE_ID);
 					}
@@ -717,16 +682,13 @@ POOL_STATUS pool_send_and_wait(POOL_QUERY_CONTEXT *query_context,
 
 		if (wait_for_query_response(frontend, CONNECTION(backend, i), MAJOR(backend)) != POOL_CONTINUE)
 		{
-			if (REPLICATION)
-			{
-				/* Cancel current transaction */
-				CancelPacket cancel_packet;
+			/* Cancel current transaction */
+			CancelPacket cancel_packet;
 
-				cancel_packet.protoVersion = htonl(PROTO_CANCEL);
-				cancel_packet.pid = MASTER_CONNECTION(backend)->pid;
-				cancel_packet.key= MASTER_CONNECTION(backend)->key;
-				cancel_request(&cancel_packet);
-			}
+			cancel_packet.protoVersion = htonl(PROTO_CANCEL);
+			cancel_packet.pid = MASTER_CONNECTION(backend)->pid;
+			cancel_packet.key= MASTER_CONNECTION(backend)->key;
+			cancel_request(&cancel_packet);
 
 			return POOL_END;
 		}
@@ -1273,10 +1235,11 @@ void where_to_send_deallocate(POOL_QUERY_CONTEXT *query_context, Node *node)
 	DeallocateStmt *d = (DeallocateStmt *)node;
 	POOL_SENT_MESSAGE *msg;
 
-	/* DEALLOCATE ALL? */
+	/* DELLOCATE ALL? */
 	if (d->name == NULL)
 	{
 		pool_setall_node_to_be_sent(query_context);
+		return;
 	}
 	else
 	{
@@ -1288,11 +1251,11 @@ void where_to_send_deallocate(POOL_QUERY_CONTEXT *query_context, Node *node)
 			/* Inherit same map from PREPARE or PARSE */
 			pool_copy_prep_where(msg->query_context->where_to_send,
 								 query_context->where_to_send);
-			return;
 		}
-		/* prepared statement was not found */
-		pool_setall_node_to_be_sent(query_context);
+		return;
 	}
+	/* prepared statement was not found */
+	pool_setall_node_to_be_sent(query_context);
 }
 
 /*
@@ -1348,8 +1311,7 @@ bool is_set_transaction_serializable(Node *node)
 		return false;
 
 	if (((VariableSetStmt *)node)->kind == VAR_SET_VALUE &&
-		(!strcmp(((VariableSetStmt *)node)->name, "transaction_isolation") ||
-                 !strcmp(((VariableSetStmt *)node)->name, "default_transaction_isolation")))
+		!strcmp(((VariableSetStmt *)node)->name, "transaction_isolation"))
 	{
 		List *options = ((VariableSetStmt *)node)->args;
 		foreach(list_item, options)
@@ -1522,6 +1484,11 @@ void pool_set_query_state(POOL_QUERY_CONTEXT *query_context, POOL_QUERY_STATE st
 	}
 }
 
+/*
+ * Return -1, 0 or 1 according to s1 is "before, equal or after" s2 in terms of state
+ * transition order. 
+ * The State transiton order is defined as: UNPARSED < PARSE_COMPLETE < BIND_COMPLETE < EXECUTE_COMPLETE
+ */
 int statecmp(POOL_QUERY_STATE s1, POOL_QUERY_STATE s2)
 {
 	int ret;
@@ -1593,75 +1560,105 @@ char* remove_read_write(int len, const char* contents, int *rewritten_len)
 }
 
 /*
- * Return true if one of followings is true
- *
- * SET transaction_read_only TO on
- * SET TRANSACTION READ ONLY
- * SET TRANSACTION CHARACTERISTICS AS TRANSACTION READ ONLY
- *
- * Note that if the node is not a variable statement, returns false.
+ * Return true if current query is safe to cache.
  */
-bool pool_is_transaction_read_only(Node *node)
+bool pool_is_cache_safe(void)
 {
-	ListCell   *list_item;
-	bool ret = false;
+	POOL_SESSION_CONTEXT *sc;
 
-	if (!IsA(node, VariableSetStmt))
-		return ret;
+	sc = pool_get_session_context();
+	if (!sc)
+		return false;
 
-	/*
-	 * SET transaction_read_only TO on
-	 */
-	if (((VariableSetStmt *)node)->kind == VAR_SET_VALUE &&
-		!strcmp(((VariableSetStmt *)node)->name, "transaction_read_only"))
+	if (pool_is_query_in_progress() && sc->query_context)
 	{
-		List *options = ((VariableSetStmt *)node)->args;
-		foreach(list_item, options)
-		{
-			A_Const *v = (A_Const *)lfirst(list_item);
-
-			switch (v->val.type)
-			{
-				case T_String:
-					if (!strcasecmp(v->val.val.str, "on") ||
-						!strcasecmp(v->val.val.str, "t") ||
-						!strcasecmp(v->val.val.str, "true"))
-						ret = true;
-					break;
-				case T_Integer:
-					if (v->val.val.ival)
-						ret = true;
-				default:
-					break;
-			}
-		}
+		return sc->query_context->is_cache_safe;
+	}
+	return false;
+}
+
+/*
+ * Set safe to cache.
+ */
+void pool_set_cache_safe(void)
+{
+	POOL_SESSION_CONTEXT *sc;
+
+	sc = pool_get_session_context();
+	if (!sc)
+		return;
+
+	if (sc->query_context)
+	{
+		sc->query_context->is_cache_safe = true;
+	}
+}
+
+/*
+ * Unset safe to cache.
+ */
+void pool_unset_cache_safe(void)
+{
+	POOL_SESSION_CONTEXT *sc;
+
+	sc = pool_get_session_context();
+	if (!sc)
+		return;
+
+	if (sc->query_context)
+	{
+		sc->query_context->is_cache_safe = false;
+	}
+}
+
+/*
+ * Return true if current temporary query cache is exceeded
+ */
+bool pool_is_cache_exceeded(void)
+{
+	POOL_SESSION_CONTEXT *sc;
+
+	sc = pool_get_session_context();
+	if (!sc)
+		return false;
+
+	if (pool_is_query_in_progress() && sc->query_context)
+	{
+		return sc->query_context->temp_cache->is_exceeded;
+	}
+	return false;
+}
+
+/*
+ * Set current temporary query cache is exceeded
+ */
+void pool_set_cache_exceeded(void)
+{
+	POOL_SESSION_CONTEXT *sc;
+
+	sc = pool_get_session_context();
+	if (!sc)
+		return;
+
+	if (sc->query_context)
+	{
+		sc->query_context->temp_cache->is_exceeded = true;
+	}
+}
+
+/*
+ * Unset current temporary query cache is exceeded
+ */
+void pool_unset_cache_exceeded(void)
+{
+	POOL_SESSION_CONTEXT *sc;
+
+	sc = pool_get_session_context();
+	if (!sc)
+		return;
+
+	if (sc->query_context)
+	{
+		sc->query_context->temp_cache->is_exceeded = false;
 	}
-
-	/*
-	 * SET SESSION CHARACTERISTICS AS TRANSACTION READ ONLY
-	 * SET TRANSACTION READ ONLY
-	 */
-	else if (((VariableSetStmt *)node)->kind == VAR_SET_MULTI &&
-			 (!strcmp(((VariableSetStmt *)node)->name, "TRANSACTION") ||
-			  !strcmp(((VariableSetStmt *)node)->name, "SESSION CHARACTERISTICS")))
-	{
-		List *options = ((VariableSetStmt *)node)->args;
-		foreach(list_item, options)
-		{
-			DefElem *opt = (DefElem *) lfirst(list_item);
-
-			if (!strcmp("transaction_read_only", opt->defname))
-			{
-				bool read_only;
-
-				read_only = ((A_Const *)opt->arg)->val.val.ival;
-				if (read_only)
-				{
-					ret = true;
-					break;
-				}
-			}
-		}
-	}
-	return ret;
 }
