@@ -6,7 +6,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2016	PgPool Global Development Group
+ * Copyright (c) 2003-2011	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -32,7 +32,6 @@ static POOL_SESSION_CONTEXT session_context_d;
 static POOL_SESSION_CONTEXT *session_context = NULL;
 
 static void init_sent_message_list(void);
-static bool can_query_context_destroy(POOL_QUERY_CONTEXT *qc);
 
 /*
  * Initialize per session context
@@ -40,8 +39,6 @@ static bool can_query_context_destroy(POOL_QUERY_CONTEXT *qc);
 void pool_init_session_context(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend)
 {
 	session_context = &session_context_d;
-	int node_id;
-	int i;
 
 	/* Get Process context */
 	session_context->process_context = pool_get_process_context();
@@ -70,23 +67,20 @@ void pool_init_session_context(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *
 	/* Choose load balancing node if neccessary */
 	if (pool_config->load_balance_mode)
 	{
-		node_id = select_load_balancing_node();
-	}
-	else
-	{
-		node_id = STREAM? PRIMARY_NODE_ID: MASTER_NODE_ID;
-	}
+		ProcessInfo *process_info = pool_get_my_process_info();
+		if (!process_info)
+		{
+			pool_error("pool_init_session_context: pool_get_my_process_info failed");
+			return;
+		}
 
-	session_context->load_balance_node_id = node_id;
+		session_context->load_balance_node_id = 
+			process_info->connection_info->load_balancing_node =
+			select_load_balancing_node();
 
-	for (i=0;i<NUM_BACKENDS;i++)
-	{
-		pool_coninfo(session_context->process_context->proc_id,
-					 pool_pool_index(), i)->load_balancing_node = node_id;
+		pool_debug("selected load balancing node: %d", backend->info->load_balancing_node);
 	}
 
-	pool_debug("selected load balancing node: %d", node_id);
-
 	/* Unset query is in progress */
 	pool_unset_query_in_progress();
 
@@ -117,6 +111,12 @@ void pool_init_session_context(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *
 	 * in UPDATE/DELETE.
 	 */
 	session_context->mismatch_ntuples = false;
+
+	if (pool_config->memory_cache_enabled)
+	{
+		session_context->query_cache_array = pool_create_query_cache_array();
+		session_context->num_selects = 0;
+	}
 }
 
 /*
@@ -129,11 +129,15 @@ void pool_session_context_destroy(void)
 		pool_clear_sent_message_list();
 		free(session_context->message_list.sent_messages);
 		pool_memory_delete(session_context->memory_context, 0);
+		if (pool_config->memory_cache_enabled)
+		{
+			pool_discard_query_cache_array(session_context->query_cache_array);
+			session_context->num_selects = 0;
+		}
 
 		if (session_context->query_context)
 			pool_query_context_destroy(session_context->query_context);
 	}
-
 	/* XXX For now, just zap memory */
 	memset(&session_context_d, 0, sizeof(session_context_d));
 	session_context = NULL;
@@ -734,13 +738,6 @@ POOL_TRANSACTION_ISOLATION pool_get_transaction_isolation(void)
 	/* No cached data is available. Ask backend. */
 	status = do_query(MASTER(session_context->backend),
 					  "SELECT current_setting('transaction_isolation')", &res, MAJOR(session_context->backend));
-	if(status != POOL_CONTINUE)
-	{
-		pool_error("pool_get_transaction_isolation: do_query returns no rows");
-		if(res)
-			free_select_result(res);
-		return POOL_UNKNOWN;
-	}
 
 	if (res->numrows <= 0)
 	{
@@ -776,8 +773,7 @@ POOL_TRANSACTION_ISOLATION pool_get_transaction_isolation(void)
 		ret = POOL_UNKNOWN;
 	}   
 
-	if(res)
-		free_select_result(res);
+	free_select_result(res);
 
 	if (ret != POOL_UNKNOWN)
 		session_context->transaction_isolation = ret;
@@ -924,7 +920,11 @@ static void init_sent_message_list(void)
 	}
 }
 
-static bool can_query_context_destroy(POOL_QUERY_CONTEXT *qc)
+/*
+ * Look for extended message list to check if given query context qc
+ * is used. Returns true if it is not used.
+ */
+bool can_query_context_destroy(POOL_QUERY_CONTEXT *qc)
 {
 	int i;
 	int count = 0;
@@ -935,11 +935,16 @@ static bool can_query_context_destroy(POOL_QUERY_CONTEXT *qc)
 	for (i = 0; i < msglist->size; i++)
 	{
 		if (msglist->sent_messages[i]->query_context == qc)
+		{
+			pool_debug("can_query_context_destroy: query context %p is still used. query:%s",
+					   qc, qc->original_query);
 			count++;
+		}
 	}
 	if (count > 1)
 	{
-		pool_debug("can_query_context_destroy: query context is still used.");
+		pool_debug("can_query_context_destroy: query context %p is still used for %d times.",
+				   qc, count);
 		return false;
 	}
 
