@@ -76,8 +76,6 @@ wd_exit(int exit_signo)
 	wd_notice_server_down();
 
 	kill (child_pid, exit_signo);
-	kill (0, exit_signo);
-
 	child_wait(0);
 
 	exit(0);
@@ -130,7 +128,7 @@ wd_main(int fork_wait_time)
 	child_pid = wd_child(1);
 	if (child_pid < 0 )
 	{
-		pool_error("lunch wd_child failed");
+		pool_error("launch wd_child failed");
 		return child_pid;
 	}
 
@@ -155,7 +153,6 @@ wd_main(int fork_wait_time)
 	signal(SIGQUIT, wd_exit);	
 	signal(SIGTERM, wd_exit);	
 	signal(SIGPIPE, SIG_IGN);	
-	setpgid(0,pgid);
 
 	set_ps_display("lifecheck",false);
 	/* wait until ready to go */
@@ -182,29 +179,32 @@ wd_chk_sticky(void)
 	
 	/* check sticky bit of ifup command */
 	wd_get_cmd(cmd, pool_config->if_up_cmd);
-	sprintf(path,"%s/%s",pool_config->ifconfig_path,cmd);
+	snprintf(path, sizeof(path), "%s/%s", pool_config->ifconfig_path, cmd);
 	if (! has_sticky_bit(path))
 	{
-		fprintf(stderr,"ifup[%s] doesn't have sticky bit\n",path);
+		pool_log("wd_chk_sticky: ifup[%s] doesn't have sticky bit", path);
 		return 0;
 	}
+
 	/* check sticky bit of ifdown command */
 	wd_get_cmd(cmd, pool_config->if_down_cmd);
-	sprintf(path,"%s/%s",pool_config->ifconfig_path,cmd);
+	snprintf(path, sizeof(path), "%s/%s", pool_config->ifconfig_path, cmd);
 	if (! has_sticky_bit(path))
 	{
-		fprintf(stderr,"ifdown[%s] doesn't have sticky bit\n",path);
+		pool_log("wd_chk_sticky: ifdown[%s] doesn't have sticky bit", path);
 		return 0;
 	}
+
 	/* check sticky bit of arping command */
 	wd_get_cmd(cmd, pool_config->arping_cmd);
-	sprintf(path,"%s/%s",pool_config->arping_path,cmd);
+	snprintf(path, sizeof(path), "%s/%s", pool_config->arping_path, cmd);
 	if (! has_sticky_bit(path))
 	{
-		fprintf(stderr,"arping[%s] doesn't have sticky bit\n",path);
+		pool_log("wd_chk_sticky: arping[%s] doesn't have sticky bit", path);
 		return 0;
 	}
-	fprintf(stderr,"all commands have sticky bit\n");
+
+	pool_log("wd_chk_sticy: all commands have sticky bit");
 	return 1;
 }
 
@@ -215,8 +215,9 @@ has_sticky_bit(char * path)
 	struct stat buf;
 	if (stat(path,&buf) < 0)
 	{
-		fprintf(stderr,"%s:stat failed\n",path);
-		return 0;
+		pool_error("has_stickey_bit: %s: no such a command", path);
+		pool_shmem_exit(1);
+		exit(1);
 	}
 	return ((buf.st_uid == 0) && (S_ISREG(buf.st_mode)) && (buf.st_mode & S_ISUID))?1:0;
 }
